
package kisthep.util;
public class Constants {



/* Chemical Constants */
	
    public static final double P0 = 1E05; // Pascal 
    
    public static final double waterMeltingTemperature = 273.15; // Kelvin
    
    public static final double roomTemperature = 298.0; // Kelvin


/* Physical constants from http://physics.nist.gov/cuu/Constants/Table/allascii.txt */

    public static final double a0 = 0.529177211E-10; // bohr radius (m) 
 
    public static final double kb = 1.380649E-23; // J.K-1 Boltzmann Cte

    public static final double NA = 6.0221413E23; //mol-1 // Avogadro Number

    public static final double R = NA * kb; // J.K-1.mol-1  

    public static final double c = 2.99792458E8 ; // m.s-1  // light celerity
    
    public static final double h = 6.6260696E-34; // J.s // Planck cte
	
    public static final double convertCm_1ToM_1=1.0E2;   // conversion cm-1 ==> m-1 

    public static final double convertCm_1ToKelvin= 100*c*h/kb;  // conversion cm-1 ==> K

    public static final double convertHartreeToJoule_perMolec= 4.3597443e-18;  // conversion hartree => Joule/molec
   
    public static final double convertHartreeToJoule=convertHartreeToJoule_perMolec*NA ; //conversion: hartree=> JOULES/MOL
    
    public static final double convertCalToJoule=4.184; //conversion: CAL => JOULE as defined by Pure and Applied Chemistry, 51 (1979) 1.
                                                        // will determine the 627.51 conversion factor (hartree to kcal/mol) !!

    public static final double convertAmuToKg=1.6605389E-27; //conversion: amu=> kg 

    public static final double convertTorrToPa = 133.322368; // Torr ==> Pa

    public static final double convertBarToPa = P0; // Bar ==> Pa


    public static final double convertAtmToPa = 1.01325*P0; // Atm ==> Pa

    public static final double convertJTocm_1 = 1.0/(kb * convertCm_1ToKelvin); // J ==> cm-1

    public static final double convertTorr_1ToCm3Molec_1Kelvin_1 = R/(convertTorrToPa * NA * 1E-6); // Torr-1 ==> molec-1.cm3.K-1
    public static final double convertJToKCalPerMol = Constants.NA*1.0E-3/convertCalToJoule; // J/molec ==> kcal/mol 
    public static final double convertGHzToAmuBohr2 = Constants.h/ (1.0E09* 8 * Math.pow(Math.PI,2)*(Constants.convertAmuToKg*Math.pow(Constants.a0,2)));
    public static final double convertcm_1ToAmuBohr2 = Constants.h/ (8 * Math.pow(Math.PI,2)*c*100*(Constants.convertAmuToKg*Math.pow(Constants.a0,2)));
     
    
 
    
/* Kisthelp Constants */
    public static final int    thermo3DChemArrayLimit = 25000;// to limit the size of the 2D array "Z" for plot3DPanel.gridPlot ...
    public static final double nthEpsilon = 10000; // the peakEnd of the function y = P(E)e^-(E/kBT) will be detected at yMax/
    public static final int    minArraySize   = 5; // the minimum number of elements of the arrays for the thermo and Kinetics properties calculations
    public static final String kisthepMessage = "K i S T h e l P ";
    public static final double inertiaEpsilon = 0.1; // in GHz (to detect identical Rotational constants)      
    public static final double massEpsilon = 0.001; // in g/mol (to detect reactants and TS of different mass)
    public static final double highEnergy = 1000.0 ; // in J/molec (to avoid hindered rotor treatment for ex.)
    
    //public static final String askingFileString="Please, enter the datafile name for ";
    public static final String askingLocationString="Enter location ";
    public static final String askingFileString="FILE: ";
    public static final double defaultScalingFactor = 1.0; // 
    public static final double nullValue = 0.0; //          
    public static final int    stateDensity = 0; 
    public static final int    stateSum = 1;

    public static final double tMinDefault = roomTemperature  ;
    public static final double tStepDefault = 1.0  ;
    public static final double tMaxDefault = tMinDefault; 
    public static final int    nStepDefault = 500;
  
    public static final double pStepDefault = 0.1*P0  ;
    
    public static final int GLPolynomiaDegree = 15;  // !! current version limited to 15-point Gauss-Laguerre integration

    public static final double MaxwBoltzMaxThresh = 1.0E-18;  //J/molec The maximum energy value allowed to find the maximum of the MB distribution function 

// allowed input files 
    public static final String g09FileType          = "gaussian09";  //gaussian 09 output
    public static final String nwcFileType          = "NWChem6.0";  //NWCHEM 6.0 output
    public static final String gms2012FileType      = "gms2012";  //Gamess output (2012 and 2013)
    public static final String kInpFileType         = "kisthepInput";  //KISTHEP input file 
    public static final String sessionFile          = "sessionFileType";  // session File Type (kisthep save file format)
    public static final String csvOutputFile        = "csvOutputFileType";  // output at excel file format

// all allowed files:
    public static final String anyAllowedDataFile = "anyAllowedData";  //any authorized input file type (kinp, gaussian, gamess, nwchem)

//  unauthorized input files for kisthep application
    public static final String unknownFile    = "unknownFile";  //unAuthorized input file type 

//  any input files
    public static final String anyFile    = "anyFile";  //any file type for any application

//  size of the main panel in the Kisthep Interface
    public static final int mainPaneWidth  = 1100; // pixels
    public static final int mainPaneHeight = 1000 ; // pixels

// system constants    
    public static String newLine = System.getProperty("line.separator");
    
}// Constants
